"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createAdapter = void 0;
var encryption_1 = require("./encryption");
var derivation_1 = require("./derivation");
var textPacking_1 = require("../shared/textPacking");
var dataPacking_1 = require("./dataPacking");
var stream_1 = require("./stream");
var symbols_1 = require("../symbols");
var types_1 = require("../types");
function createAdapter() {
    var adapter = {
        algorithm: symbols_1.ALGO_DEFAULT,
        createDecryptStream: function (password) { return (0, stream_1.createDecryptStream)(adapter, password); },
        createEncryptStream: function (password) { return (0, stream_1.createEncryptStream)(adapter, password); },
        decrypt: function (encrypted, password) { return decrypt(adapter, encrypted, password); },
        decryptCBC: encryption_1.decryptCBC,
        decryptGCM: encryption_1.decryptGCM,
        derivationRounds: symbols_1.DERIVED_KEY_ITERATIONS,
        deriveKey: function (password, salt) { return deriveKey(adapter, password, salt); },
        encrypt: function (text, password) { return encrypt(adapter, text, password); },
        encryptCBC: encryption_1.encryptCBC,
        encryptGCM: encryption_1.encryptGCM,
        generateIV: encryption_1.generateIV,
        generateSalt: encryption_1.generateSalt,
        packData: dataPacking_1.packEncryptedData,
        packText: textPacking_1.packEncryptedText,
        setAlgorithm: function (algo) {
            adapter.algorithm = algo;
            return adapter;
        },
        setDerivationRounds: function (rounds) {
            adapter.derivationRounds = rounds;
            return adapter;
        },
        unpackData: dataPacking_1.unpackEncryptedData,
        unpackText: textPacking_1.unpackEncryptedText
    };
    return adapter;
}
exports.createAdapter = createAdapter;
function decrypt(adapter, encrypted, password) {
    return __awaiter(this, void 0, void 0, function () {
        var encryptedComponents, salt, rounds, method, decryptData, keyDerivationInfo;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    encryptedComponents = typeof encrypted === "string"
                        ? adapter.unpackText(encrypted)
                        : adapter.unpackData(encrypted);
                    salt = encryptedComponents.salt, rounds = encryptedComponents.rounds, method = encryptedComponents.method;
                    decryptData = getDecryptionMethod(adapter, method);
                    adapter.algorithm = method;
                    adapter.derivationRounds = rounds;
                    return [4 /*yield*/, adapter.deriveKey(password, salt)];
                case 1:
                    keyDerivationInfo = _a.sent();
                    return [2 /*return*/, decryptData(encryptedComponents, keyDerivationInfo)];
            }
        });
    });
}
function deriveKey(adapter, password, salt) {
    return __awaiter(this, void 0, void 0, function () {
        var algorithm, derivationRounds;
        return __generator(this, function (_a) {
            algorithm = adapter.algorithm, derivationRounds = adapter.derivationRounds;
            return [2 /*return*/, (0, derivation_1.deriveKeyFromPassword)(password, salt, derivationRounds, hmacKeyRequired(algorithm))];
        });
    });
}
function encrypt(adapter, text, password) {
    return __awaiter(this, void 0, void 0, function () {
        var algorithm, encryptData, salt, _a, keyDerivationInfo, iv, encryptedComponents;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    algorithm = adapter.algorithm;
                    encryptData = getEncryptionMethod(adapter, algorithm);
                    return [4 /*yield*/, adapter.generateSalt(symbols_1.SALT_LENGTH)];
                case 1:
                    salt = _b.sent();
                    return [4 /*yield*/, Promise.all([
                            adapter.deriveKey(password, salt),
                            adapter.generateIV()
                        ])];
                case 2:
                    _a = _b.sent(), keyDerivationInfo = _a[0], iv = _a[1];
                    return [4 /*yield*/, encryptData(text, keyDerivationInfo, iv)];
                case 3:
                    encryptedComponents = _b.sent();
                    return [2 /*return*/, typeof text === "string"
                            ? adapter.packText(encryptedComponents)
                            : adapter.packData(encryptedComponents)];
            }
        });
    });
}
function getDecryptionMethod(adapter, algo) {
    if (algo === types_1.EncryptionAlgorithm.CBC) {
        return adapter.decryptCBC;
    }
    else if (algo === types_1.EncryptionAlgorithm.GCM) {
        return adapter.decryptGCM;
    }
    throw new Error("Invalid algorithm: ".concat(algo));
}
function getEncryptionMethod(adapter, algo) {
    if (algo === types_1.EncryptionAlgorithm.CBC) {
        return adapter.encryptCBC;
    }
    else if (algo === types_1.EncryptionAlgorithm.GCM) {
        return adapter.encryptGCM;
    }
    throw new Error("Invalid algorithm: ".concat(algo));
}
function hmacKeyRequired(algo) {
    return algo === types_1.EncryptionAlgorithm.GCM ? false : true;
}
